/*
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * Copyright (C) 2007 Dr. Ruediger Berlich - ruediger@berlich.de
 */

// Compile with "g++ -o random1 random1.cpp"
// Tested on OpenSUSE 10.3, g++ 4.21, boost 1.34.1 . Note that, by
// default, OpenSUSE 10.3 still comes with boost 1.33.1 . In order to 
// compile Boost 1.34.1 on OpenSUSE 10.3, you might have to add the 
// compiler switch  -fno-strict-aliasing in line 155 of the file 
// $BOOSTHOME/tools/jam/src/build.jam .

#include <iostream>
#include <iomanip>
#include <ctime>

#include <boost/random.hpp>

using namespace std;
using namespace boost;

int main(int argc, char** argv){   
  // Mersenne Twister
  mt19937 mersenne(static_cast<unsigned int> (time(0)));
  // Linear Congruential
  minstd_rand0 linearCongruential(static_cast<unsigned int> (time(0)));

  // Uniform distribution
  uniform_real<double> evenDist(0.,7.); // Attention: possibly buggy according to the documentation
  // Normal/Gaussian distribution
  normal_distribution<double> normalDist(0.,1.); 

  // Combination of generators with distributions
  variate_generator<mt19937&, uniform_real<double> > even(mersenne, evenDist);
  variate_generator<minstd_rand0&, normal_distribution<double> > gauss(linearCongruential, normalDist);

  // Output
  cout.precision(2);

  cout << "Even distribution [0,7[ :" << endl;
  for(unsigned int i=0; i<10; i++){
    cout << fixed << even() << " ";
  }
  cout << endl;

  cout << "Normal distribution with mean 0 and sigma 1 :" << endl;
  for(unsigned int i=0; i<10; i++){
    cout << gauss() << " ";
  }
  cout << endl;

  return 0;
}
