/*
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * Copyright (C) 2007 Dr. Ruediger Berlich - ruediger@berlich.de
 */

// Compile with "g++ -lboost_serialization-gcc42-mt -o serialization0 serialization0.cpp"
// Tested on OpenSUSE 10.3, g++ 4.21, boost 1.34.1 . Note that, by
// default, OpenSUSE 10.3 still comes with boost 1.33.1 . In order to 
// compile Boost 1.34.1 on OpenSUSE 10.3, you might have to add the 
// compiler switch  -fno-strict-aliasing in line 155 of the file 
// $BOOSTHOME/tools/jam/src/build.jam .

#include <iostream>
#include <string>
#include <vector>
#include <string>
#include <sstream>
#include <ctime>

#include <boost/function.hpp>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/random.hpp>
#include <boost/archive/xml_oarchive.hpp>
#include <boost/archive/xml_iarchive.hpp>
#include <boost/archive/text_oarchive.hpp>
#include <boost/archive/text_iarchive.hpp>
#include <boost/serialization/nvp.hpp>
#include <boost/serialization/vector.hpp>
#include <boost/serialization/shared_ptr.hpp>
#include <boost/serialization/base_object.hpp>
#include <boost/serialization/utility.hpp>

using namespace std;
using namespace boost;
using namespace boost::serialization;
using namespace boost::archive;

/******************************************************************************************/

int main(int argc, char **argv){ 
  vector<int> simple;
  // Fill vector<int> with values
  for(int i=0; i<3; i++) simple.push_back(i);
  
  // Serialize
  ostringstream oss;
  {
    xml_oarchive oa(oss);
    oa << make_nvp("simple",simple);
  }

  // Let the audience know
  cout << "XML description of \"simple:\"" << endl << endl
       << oss.str() << endl;

  // Fill XML code back into another vector<int>
  vector<int> simple2;
  istringstream iss(oss.str());
  {
    xml_iarchive ia(iss);
    ia >> make_nvp("simple",simple2);
  }

  // Show the content of the new object
  cout << "Content of \"simple2:\"" << endl;
  for(int i=0; i<simple2.size(); i++) cout << simple2[i] << " ";
  cout << endl;
}

/****************************************************************/
